from __future__ import absolute_import

import random
import datetime
import uuid


class DjipsumFields(object):
    """
    Define all about fields.

    usage:
        DjipsumFields(model_class, fields, maximum).create_validated_fields()
    """

    def __init__(self, model_class, fields, maximum):
        self.model_class = model_class
        self.fields = fields
        self.maximum = maximum

    def randomize(self, list):
        """
        Return the random choice.
        """
        return random.choice(list)

    def randomBinaryField(self):
        """
        Return random bytes format.
        """
        lst = [
            b"hello world",
            b"this is bytes",
            b"awesome django",
            b"djipsum is awesome",
            b"\x00\x01\x02\x03\x04\x05\x06\x07",
            b"\x0b\x0c\x0e\x0f"
        ]
        return self.randomize(lst)

    def randomCharField(self, model_class, field_name):
        """
        Checking if `field_name` has choices.
        Then, returning random value from it.
        Result of: `available_choices`
        [
          ('project', 'I wanna to talk about project'),
          ('feedback', 'I want to report a bugs or give feedback'),
          ('hello', 'I just want to say hello')
        ]
        """
        try:
            available_choices = model_class._meta.get_field(field_name).get_choices()[1:]
            return self.randomize([ci[0] for ci in available_choices])

        except AttributeError:
            lst = [
                "Enthusiastically whiteboard synergistic methods",
                "Authoritatively scale progressive meta-services through",
                "Objectively implement client-centered supply chains via stand-alone",
                "Phosfluorescently productize accurate products after cooperative results",
                "Appropriately drive cutting-edge systems before optimal scenarios",
                "Uniquely productize viral ROI for competitive e-markets"
                "Uniquely repurpose high-quality models vis-a-vis",
                "Django is Fucking Awesome? Yes"
            ]
            return self.randomize(lst)

    def randomCommaSeparatedIntegerField(self):
        """
        Return the unique integers in the string such as below:
            '6,1,7' or '4,5,1,3,2' or '2,7,9,3,5,4,1' or '3,9,2,8,7,1,5,4,6'
        """
        randint = lambda max: ",".join(
            [str(x) for x in random.sample(range(1, 10), max)]
        )
        lst = [
            randint(3),
            randint(5),
            randint(7),
            randint(9)
        ]
        return self.randomize(lst)

    def randomDecimalField(self, model_class, field_name):
        """
        Validate if the field has a `max_digits` and `decimal_places`
        And generating the unique decimal number.
        """
        decimal_field = model_class._meta.get_field(field_name)
        max_digits = None
        decimal_places = None

        if decimal_field.max_digits is not None:
            max_digits = decimal_field.max_digits
        if decimal_field.decimal_places is not None:
            decimal_places = decimal_field.decimal_places

        digits = random.choice(range(100))
        if max_digits is not None:
            start = 0
            if max_digits < start:
                start = max_digits - max_digits
            digits = int(
                "".join([
                    str(x) for x in random.sample(
                        range(start, max_digits),
                        max_digits - 1
                    )
                ])
            )
        places = random.choice(range(10, 99))
        if decimal_places is not None:
            places = str(
                random.choice(range(9999 * 99999))
            )[:decimal_places]

        return float(
            str(digits)[:decimal_places] + "." + str(places)
        )

    def randomEmailField(self):
        """
        Return the random string emails.
        """
        lst = [
            "admin@gmail.com",
            "sample@gmail.com",
            "django@site.com",
            "test@site.com",
        ]
        return self.randomize(lst)

    def randomFileField(self):
        """
        Return the random string files.
        """
        lst = [
            "file.zip",
            "awesomefile.tar.gz",
            "samplefile.docx",
            "djipsum.pdf"
        ]
        return self.randomize(lst)

    def randomImageField(self):
        """
        Return string images.
        """
        lst = [
            "avatar.jpg",
            "djipsum.jpeg",
            "sampleimage.png",
            "awesome_django.gif"
        ]
        return self.randomize(lst)

    def randomGenericIPAddressField(self):
        """
        Return string ipv4 or ipv6
        """
        lst = [
            "192.168.1.1",  # ipv4
            "66.249.65.54",  # public
            "255.255.255.0",  # netmask
            "2001:db8:a0b:12f0::1",  # ipv6
        ]
        return self.randomize(lst)

    def randomSlugField(self):
        """
        Return the unique slug by generating the uuid4
        to fix the duplicate slug (unique=True)
        """
        lst = [
            "sample-slug-{}".format(uuid.uuid4().hex),
            "awesome-djipsum-{}".format(uuid.uuid4().hex),
            "unique-slug-{}".format(uuid.uuid4().hex)
        ]
        return self.randomize(lst)

    def randomTextField(self):
        """
        Return the string paragraph.
        """
        lst = [
            "Competently coordinate real-time content without effective functionalities. "
            "Rapidiously extend goal-oriented process improvements rather than empowered strategic theme areas. "
            "Collaboratively disintermediate equity invested ideas through resource maximizing architectures. "
            "Synergistically facilitate clicks-and-mortar methodologies through resource sucking communities. "
            "Appropriately drive cutting-edge systems before optimal scenarios.",

            "Conveniently facilitate best-of-breed experiences via integrated web-readiness. "
            "Energistically parallel task interdependent leadership skills with cross-media experiences. "
            "Dramatically morph emerging content and web-enabled infomediaries. "
            "Quickly facilitate excellent manufactured products after open-source 'outside the box' thinking. "
            "Dramatically customize enterprise-wide models whereas resource-leveling total linkage. ",

            "Interactively benchmark efficient leadership skills whereas clicks-and-mortar partnerships. "
            "Conveniently incentivize intermandated e-tailers whereas installed base niche markets. "
            "Globally implement scalable processes before multidisciplinary users. "
            "Phosfluorescently productize accurate products after cooperative results.",

            "Objectively strategize error-free metrics through efficient materials. "
            "Enthusiastically innovate cooperative niches before client-centric models. "
            "Intrinsicly exploit user-centric value after state of the art synergy. "
            "Rapidiously foster team building materials for interoperable applications. "
            "Distinctively fabricate pandemic deliverables whereas focused interfaces. "
            "Uniquely productize viral ROI for competitive e-markets."
        ]
        return self.randomize(lst)

    def randomURLField(self):
        """
        Return the string url.
        """
        lst = [
            "https://djangoproject.com",
            "https://python.web.id",
            "http://sample.com",
            "https://google.com",
            "http://stackoverflow.com"
        ]
        return self.randomize(lst)

    def randomUUIDField(self):
        """
        Return the unique uuid from uuid1, uuid3, uuid4, or uuid5.
        """
        uuid1 = uuid.uuid1().hex
        uuid3 = uuid.uuid3(
            uuid.NAMESPACE_URL,
            self.randomize(['python', 'django', 'awesome'])
        ).hex
        uuid4 = uuid.uuid4().hex
        uuid5 = uuid.uuid5(
            uuid.NAMESPACE_DNS,
            self.randomize(['python', 'django', 'awesome'])
        ).hex
        return self.randomize([uuid1, uuid3, uuid4, uuid5])

    def getOrCreateForeignKey(self, model_class, field_name):
        """
        Return related random object to set as ForeignKey.
        """
        # Getting related object type
        # Eg: <django.db.models.fields.related.ForeignKey: test_ForeignKey>
        instance = getattr(model_class, field_name).field

        # Getting the model name by instance to find/create first id/pk.
        # Eg: <class 'django.contrib.auth.models.User'>
        related_model = instance.related_model().__class__

        # Trying to get random id from queryset.
        objects = related_model.objects.all()
        if objects.exists():
            return self.randomize(objects)

        # Returning first object from tuple `(<User: user_name>, False)`
        return related_model.objects.get_or_create(pk=1)[0]

    def create_validated_fields(self):
        """
        To generate lorem ipsum by validated fields for the model.
        """
        model_class = self.model_class
        fields = self.fields
        maximum = self.maximum
        objects = []

        for n in range(maximum):
            data_dict = {}
            for field in fields:

                def default_assign(func):
                    data_dict[field['field_name']] = func

                def string_assign(func):
                    data_dict[field['field_name']] = str(func)

                if field['field_type'] == 'BigIntegerField':  # values from -9223372036854775808 to 9223372036854775807
                    default_assign(random.randint(-9223372036854775808, 9223372036854775807))
                elif field['field_type'] == 'BinaryField':  # b'', self.randomBinaryField()
                    default_assign(self.randomBinaryField())
                elif field['field_type'] == 'BooleanField':  # True/False
                    default_assign(self.randomize([True, False]))
                elif field['field_type'] == 'CharField':  # self.randomCharField()
                    string_assign(self.randomCharField(model_class, field['field_name']))
                elif field['field_type'] == 'CommaSeparatedIntegerField':  # self.randomCommaSeparatedIntegerField()
                    string_assign(self.randomCommaSeparatedIntegerField())
                elif field['field_type'] == 'DateField':  # '2016-10-11'
                    string_assign(str(datetime.datetime.now().date()))
                elif field['field_type'] == 'DateTimeField':  # '2016-10-11 00:44:08.864285'
                    string_assign(str(datetime.datetime.now()))
                elif field['field_type'] == 'DecimalField':  # self.randomDecimalField()
                    default_assign(self.randomDecimalField(model_class, field['field_name']))
                elif field['field_type'] == 'DurationField':  # such as 1 day, 4 days or else.
                    default_assign(datetime.timedelta(days=random.randint(1, 10)))
                elif field['field_type'] == 'EmailField':  # self.randomEmailField()
                    string_assign(self.randomEmailField())
                elif field['field_type'] == 'FileField':  # self.randomFileField()
                    string_assign(self.randomFileField())
                elif field['field_type'] == 'FloatField':  # 1.92, 0.0, 5.0, or else.
                    default_assign(float(("%.2f" % float(random.randint(0, 100) / 13))))
                elif field['field_type'] == 'ImageField':  # self.randomImageField()
                    string_assign(self.randomImageField())
                elif field['field_type'] == 'IntegerField':  # values from -2147483648 to 2147483647
                    default_assign(random.randint(-2147483648, 2147483647))
                elif field['field_type'] == 'GenericIPAddressField':  # self.randomGenericIPAddressField()
                    string_assign(self.randomGenericIPAddressField())
                elif field['field_type'] == 'NullBooleanField':  # by Default is None/null
                    default_assign(self.randomize([None, True, False]))
                elif field['field_type'] == 'PositiveIntegerField':  # values from 0 to 2147483647
                    default_assign(random.randint(0, 2147483647))
                elif field['field_type'] == 'PositiveSmallIntegerField':  # values from 0 to 32767
                    default_assign(random.randint(0, 32767))
                elif field['field_type'] == 'SlugField':  # self.randomSlugField()
                    string_assign(self.randomSlugField())
                elif field['field_type'] == 'SmallIntegerField':  # values from -32768 to 32767
                    default_assign(random.randint(-32768, 32767))
                elif field['field_type'] == 'TextField':  # self.randomTextField()
                    string_assign(self.randomTextField())
                elif field['field_type'] == 'TimeField':  # accepts the same as DateField
                    string_assign(str(datetime.datetime.now().date()))
                elif field['field_type'] == 'URLField':  # self.randomURLField()
                    string_assign(self.randomURLField())
                elif field['field_type'] == 'UUIDField':  # self.randomUUIDField()
                    string_assign(self.randomUUIDField())
                elif field['field_type'] == 'ForeignKey':  # self.getOrCreateForeignKey()
                    default_assign(self.getOrCreateForeignKey(model_class, field['field_name']))
                # elif field['field_type'] == 'OneToOneField':  # pk/id -> not fixed yet.
                #    default_assign(self.randomize([1, ]))
                # Unsolved: need specific pk/id

            obj = model_class.objects.create(**data_dict)

            # Because the Relationship Model need specific id from the object,
            # so, i handle it after created the object.
            for field in fields:
                if field['field_type'] == 'ManyToManyField':
                    # Find the instance model field from `obj` already created before.
                    instance_m2m = getattr(obj, field['field_name'])
                    objects_m2m = instance_m2m.model.objects.all()

                    # Djipsum only create the `ManyToManyField` if the related object is exists.
                    if objects_m2m.exists():
                        ids_m2m = [i.pk for i in objects_m2m]
                        random_decission = random.sample(
                            range(min(ids_m2m), max(ids_m2m)), max(ids_m2m) - 1
                        )
                        # Let me know if the `random_decission` has minimum objects to be choice.
                        if len(random_decission) <= 2:
                            random_decission = [self.randomize(ids_m2m)]
                        related_objects = [
                            rel_obj for rel_obj in objects_m2m
                            if rel_obj.pk in random_decission
                        ]
                        # adding the `ManyToManyField`
                        instance_m2m.add(*related_objects)
            try:
                obj.save_m2m()
            except:
                obj.save()
            objects.append(obj)
        return objects



from __future__ import absolute_import

import random
from django.apps import apps
from djipsum.fields import DjipsumFields
from faker import Faker


class FakerModel(object):
    """
    Example Usage:

    - Basic API
        >>> from djipsum.faker import FakerModel
        >>> faker = FakerModel(app='app_name', model='ModelName')
        >>> faker.fake_email() # From Djipsum
        'admin@gmail.com'
        >>> faker.fake.email() # From Faker Factory
        'smithadrian@hotmail.com'
        >>>

    - Example API Usage
        >>> from djipsum.faker import FakerModel
        >>> faker = FakerModel(app='app_blog', model='Post')
        >>>
        >>> for _ in range(2):
        ...     fields = {
        ...         'user': faker.fake_relations(
        ...             type='fk',
        ...             field_name='user'
        ...         ),
        ...         'title': faker.fake.text(max_nb_chars=100),
        ...         'slug': faker.fake.slug(
        ...             faker.fake.text(max_nb_chars=50)
        ...         ),
        ...         'categories': faker.fake_relations(
        ...             type='m2m',
        ...             field_name='categories'
        ...         ),
        ...         'description': ' '.join(faker.fake.paragraphs()),
        ...         'created': str(faker.fake.date_time()),
        ...         'publish': faker.fake_boolean(),
        ...     }
        ...     faker.create(fields)
        ...
        <Post: Sit sunt nam aperiam ratione consequatur. Animi cupiditate atque totam.>
        <Post: Tempora porro sint quasi nisi totam doloremque repellat. Ducimus nesciunt impedit animi.>
        >>>
    """

    fake = Faker()

    def __init__(self, app, model):
        """
        :param `app` : is app name for `AppConfig`
        :param `model`: is model class name, e.g: Post, Profile, Tag, etc.
        """
        self.app = app
        self.model = model

    def model_class(self):
        model_class = apps.get_model(
            app_label=self.app,
            model_name=self.model
        )
        return model_class

    def djipsum_fields(self):
        """
        Extended from `DjipsumFields` for some special fields.
        """
        return DjipsumFields(self.model_class(), None, None)

    def fake_binary(self):
        """
        Return random binary format.
        Faker Factory also provide about this binary.

        Example:
            b"\x00\x01\x02\x03\x04\x05\x06\x07"
            b"\x0b\x0c\x0e\x0f"

        1. from Djipsum
            faker.fake_binary()
        2. from Faker Factory
            faker.fake.binary(length=10)
        """
        return self.djipsum_fields().randomBinaryField()

    def fake_chars_or_choice(self, field_name):
        """
        Return fake chars or choice it if the `field_name` has choices.
        Then, returning random value from it.
        This specially for `CharField`.

        Example for field:
            TYPE_CHOICES = (
              ('project', 'I wanna to talk about project'),
              ('feedback', 'I want to report a bugs or give feedback'),
              ('hello', 'I just want to say hello')
            )
        """
        return self.djipsum_fields().randomCharField(
            self.model_class(),
            field_name=field_name
        )

    def fake_comma_separated_integer(self):
        """
        Return the unique integers in the string.
        This specially for `CommaSeparatedIntegerField`.

        Example:
            '6,1,7' or '4,5,1,3,2' or '2,7,9,3,5,4,1'
        """
        return self.djipsum_fields().randomCommaSeparatedIntegerField()

    def fake_decimal(self):
        """
        Validate if the field has a `max_digits` and `decimal_places`
        And generating the unique decimal number.

        Example:
            10.7, 13041.00, 200.000.000
        """
        return self.djipsum_fields().randomDecimalField(
            self.model_class(),
            field_name=field_name
        )

    def fake_boolean(self):
        """
        Example:
            True, False
        """
        return self.djipsum_fields().randomize([True, False])

    def fake_null_boolean(self):
        """
        Faker Factory also provide about this null boolean.

        Example:
            None, True, False

        1. from Djipsum
            faker.fake_null_boolean()
        2. from Faker Factory
            faker.fake.null_boolean()
        """
        return self.djipsum_fields().randomize([None, True, False])

    def fake_float(self):
        """
        Example:
            0.69, 20.55, 98.12
        """
        return float(("%.2f" % float(random.randint(0, 100) / 13)))

    def fake_email(self):
        """
        Faker Factory also provide about this email.

        Example:
            agus@python.web.id, sample@gmail.com, hello@yahoo.com

        1. from Djipsum
            faker.fake_email()
        2. from Faker Factory
            faker.fake.email()
        """
        return self.djipsum_fields().randomEmailField()

    def fake_file(self):
        """
        Return string name of file.
        Faker Factory also provide about this file.

        Example:
            file.zip, awesomefile.tar.gz, samplefile.docx, djipsum.pdf

        1. from Djipsum
            faker.fake_file()
        2. from Faker Factory
            faker.fake.file_name()
        """
        return self.djipsum_fields().randomFileField()

    def fake_image(self):
        """
        Return string name of image.
        Example:
            avatar.jpg, djipsum.jpeg, sampleimage.png, awesome_django.gif
        """
        return self.djipsum_fields().randomImageField()

    def fake_ipaddress(self):
        """
        Faker Factory also provide about this ipaddress,
        such as ipv4, ipv6, ...etc

        Example:
            192.168.1.1, 66.249.65.54, 255.255.255.0, 2001:db8:a0b:12f0::1

        1. from Djipsum
            faker.fake_ipaddress()
        2. from Faker Factory
            faker.fake.ipv4(), faker.fake.ipv6()
        """
        return self.djipsum_fields().randomGenericIPAddressField()

    def fake_slug(self):
        """
        Optionall unique slug with uuid.
        Faker Factory also provide about this slug.

        Example:
        1. from Djipsum
            faker.fake_slug()
        2. from Faker Factory
            faker.fake.slug(
                faker.fake.text(max_nb_chars=50)
            )
        """
        return self.djipsum_fields().randomSlugField()

    def fake_paragraphs(self):
        """
        Generate the paragraphs for `TextField`.
        Faker Factory also provide about this paragraphs.

        Example:
        1. from Djipsum
            faker.fake_paragraphs()
        2. from Faker Factory
            ' '.join(faker.fake.paragraphs())
        """
        return self.djipsum_fields().randomTextField()

    def fake_url(self):
        """
        Generate the url for `URLField`.
        Faker Factory also provide about this url.

        Example:
        1. from Djipsum
            faker.fake_url()
        2. from Faker Factory
            faker.fake.url()
        """
        return self.djipsum_fields().randomURLField()

    def fake_uuid(self):
        """
        Generate the unique uuid
        from uuid1, uuid3, uuid4, or uuid5.
        """
        return self.djipsum_fields().randomUUIDField()

    def fake_relations(self, type, field_name):
        """
        Return the dictionary of object/s relation
        to process the Relationship.

        Example Output:
            {'type': 'fk', 'field_name': 'author'}, or
            {'type': 'm2m', 'field_name': 'categories'}
        """
        return {
            'type': type.lower(),
            'field_name': field_name
        }

    def fake_fk(self, field_name):
        """
        Return related random object to set as ForeignKey.

        Example Output:
            <User: username>
        """
        return self.djipsum_fields().getOrCreateForeignKey(
            model_class=self.model_class(),
            field_name=field_name
        )

    def fake_m2m(self, obj, field_name):
        """
        Return the random objects from m2m relationship.
        The ManyToManyField need specific object,
        so i handle it after created the object.
        """
        instance_m2m = getattr(obj, field_name)
        objects_m2m = instance_m2m.model.objects.all()

        if objects_m2m.exists():
            ids_m2m = [i.pk for i in objects_m2m]
            random_decission = random.sample(
                range(min(ids_m2m), max(ids_m2m)), max(ids_m2m) - 1
            )
            if len(random_decission) <= 2:
                random_decission = [
                    self.djipsum_fields().randomize(ids_m2m)
                ]
            related_objects = [
                rel_obj for rel_obj in objects_m2m
                if rel_obj.pk in random_decission
            ]
            instance_m2m.add(*related_objects)

    def create(self, fields):
        """
        Create the object only once.
        So, you need loop to usage.

        :param `fields` is dictionary fields.
        """
        try:
            # Cleaning the fields, and check if has `ForeignKey` type.
            cleaned_fields = {}
            for key, value in fields.items():
                if type(value) is dict:
                    try:
                        if value['type'] == 'fk':
                            fake_fk = self.fake_fk(value['field_name'])
                            cleaned_fields.update({key: fake_fk})
                    except:
                        pass
                else:
                    cleaned_fields.update({key: value})

            # Creating the object from dictionary fields.
            model_class = self.model_class()
            obj = model_class.objects.create(**cleaned_fields)

            # The `ManyToManyField` need specific object,
            # so i handle it after created the object.
            for key, value in fields.items():
                if type(value) is dict:
                    try:
                        if value['type'] == 'm2m':
                            self.fake_m2m(obj, value['field_name'])
                    except:
                        pass
            try:
                obj.save_m2m()
            except:
                obj.save()
            return obj
        except Exception as e:
            raise e



from __future__ import absolute_import

from django.apps import apps
from django.core.management.base import BaseCommand, CommandError

import sys
from djipsum import __VERSION__
from djipsum.fields import DjipsumFields


class Command(BaseCommand):
    help = 'To generate awesome lorem ipsum for your model!'

    def add_arguments(self, parser):
        parser.add_argument(
            '-dv',
            '--djipsum_version',
            action='store_true',
            help='Show djipsum version number and exit.'
        )
        parser.add_argument(
            '-auto',
            '--auto_gen',
            action='store_true',
            help='Automatic generate lorem ipsum from custom generator class.'
        )
        parser.add_argument(
            '-cg',
            '--custom_generator',
            help='Custom a function generator (full path) for auto-gen.'
        )
        parser.add_argument(
            '--app',
            help='The app name.'
        )
        parser.add_argument(
            '--model',
            help='The model class name.'
        )
        parser.add_argument(
            '--max',
            type=int,
            default=10,
            help='Maximum generate lorem ipsum.'
        )

    def handle(self, *args, **options):
        auto_gen = options['auto_gen']
        custom_generator = options['custom_generator']
        app = options['app']
        model = options['model']
        maximum = options['max']

        if auto_gen and custom_generator:
            components = custom_generator.split('.')
            func_name = components[-1]
            try:
                mod = __import__('.'.join(components[:-1]), fromlist=[func_name])
                generate_cst_faker = getattr(mod, func_name)
                self.stdout.write(
                    self.style.SUCCESS(
                        generate_cst_faker(maximum=maximum)
                    )
                )
                sys.exit()
            except Exception as e:
                raise CommandError(e)

        elif options['djipsum_version']:
            return __VERSION__
        elif app == None:
            return self.print_help('djipsum', '-h')
        try:
            model_class = apps.get_model(app_label=app, model_name=model)
        except Exception as e:
            raise CommandError(e)

        exclude = ['pk', 'id', 'objects']
        # removing `model_class._meta.fields` to `model_class._meta.get_fields()
        # to get all fields.`
        # Ref: http://stackoverflow.com/a/3106314/6396981
        fields = [
            {'field_type': f.__class__.__name__, 'field_name': f.name}
            for f in model_class._meta.get_fields() if f.name not in exclude
        ]
        validated_model_fields = DjipsumFields(
            model_class,
            fields,
            maximum
        ).create_validated_fields()

        def loremInfo():
            return """\n[+] Successfully generate the lorem ipsum for `{0}`\n\n{1}\n""".format(
                model_class,
                validated_model_fields
            )

        self.stdout.write(
            self.style.SUCCESS(
                loremInfo()
            )
        )
